/* For license details see bottom.
 * Copyright (c) 2002 Catalyst of Design (David Morris-Oliveros).  All rights reserved.
 */

// system includes
#include <caosGL/core/globals.h>
#include <caosGL/core/types.h>

// package includes
#include <caosGL/gfx/cScene.h>

// extern includes
#include <caosGL/core/cParser.h>
#include <caosGL/core/Dyngl.h>
#include <caosGL/core/math.h>
#include <caosGL/core/cGLTextureSpace.h>
#include <caosGL/core/cGLTexture.h>
#include <caosGL/gfx/cSurfaceProvider.h>
#include <caosGL/gfx/cState.h>
#include <GL/glu.h>

using namespace caosGL::gfx;
using namespace caosGL::core;

#define cTestEffect_attribsFile <caosGL/effects/cTestEffect.attribs>

namespace caosGL {
	namespace effects {
		static vector <tAttr> attributeNames;
		static log4cpp::Category& cat = log4cpp::Category::getInstance ("caosGL::effects::cTestEffect");

		/**
		 *<br> class:		cTestEffect
		 *<br> namespace:	caosGL::effects
		 *<br> inherits:	caosGL::gfx::cScene
		 *<br> implements:	<none>
		 *<br> purpose:		Generic test center for effects. Can remember the last thing i asked it to do...
		 *
		 */

		class CAOSGL_API cTestEffect : public cScene {
			typedef cScene super;
		private:
			#define ATTRIB(n,t,v,d) ATTRIB_DEFINE_VAR(n,t)
			#include cTestEffect_attribsFile
			#undef ATTRIB
			cGLTexture * _img;

		public:
			/********************************************************************************************/
			cTestEffect (const string n, cGroup * f) : super (n,f) {
				init ();
			}

			/********************************************************************************************/
			~cTestEffect () {
			}

			// from iDrawable
			/********************************************************************************************/
			tBool draw (const tFloat time) {
				if (!_img)
					return false;
				_img->prepare ();
				tBool restoreDepthTest = cState::isEnabled (cState::oGL_DEPTH_TEST)?true:false;
				GLboolean restoreDepthWrite; 
				glGetBooleanv (GL_DEPTH_WRITEMASK, &restoreDepthWrite);
				glDepthMask (false);
				cState::disable (cState::oGL_DEPTH_TEST);

				glMatrixMode(GL_PROJECTION);
				glPushMatrix ();
				glLoadIdentity();
				gluPerspective (45.0,1.3,0.001,100.0);
				glTranslatef (0, 0, -2.6);

				glMatrixMode(GL_MODELVIEW);
				glPushMatrix ();

				glLoadIdentity();
				
				glColor4f (1, 1, 1, _alph);
				cState::disable (cState::oGL_LIGHTING);
				cState::enable (cState::oGL_TEXTURE_2D);
				cState::enable (cState::oGL_BLEND);
				glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
				glTranslatef (_tX, _tY, 0);
				glScalef     (_sX, _sY, 1);
				glRotatef (_rX, 1, 0, 0);
				glRotatef (_rY, 0, 1, 0);
				glBegin (GL_QUADS);
					glTexCoord2f (0.99f, 0.01f); glVertex3f ( 1.0f, -1.0f,  0.0f);
					glTexCoord2f (0.01f, 0.01f); glVertex3f (-1.0f, -1.0f,  0.0f);
					glTexCoord2f (0.01f, 0.99f); glVertex3f (-1.0f,  1.0f,  0.0f);
					glTexCoord2f (0.99f, 0.99f); glVertex3f ( 1.0f,  1.0f,  0.0f);
				glEnd ();

				if (restoreDepthTest)
					cState::enable (cState::oGL_DEPTH_TEST);
				glDepthMask (restoreDepthWrite);

				glPopMatrix ();
				glMatrixMode(GL_PROJECTION);
				glPopMatrix ();

				return true;
			}

			// from iDrawable
			/********************************************************************************************/
			tBool transparent () const {
				return _alph<1;
			}

			// from iDrawable
			/********************************************************************************************/
			tVoid transparent (const tBool trasnp) {
			}

			// from cBaseNode
			/********************************************************************************************/
			tBool compile () {
				if (_tex == "") {
					cat.error ("I must know the texture name at scene compile time");
					return false;
				}
				_img = cGLTextureSpace::instance ()->get (_tex);
				return true;
			}

			// from cBaseNode
			/********************************************************************************************/
			tBool visit (tFloat t) {
				if (!cBaseNode::visit (t))
					return false;
				evaluateAll (t);

				return true;
			}

			// from cBaseNode
			/********************************************************************************************/
			tVoid leave () {
				return;
			}

			// from cBaseNode
			/********************************************************************************************/
			tBool init () {
				#define ATTRIB(n,t,v,d) ATTRIB_INIT_VAR(n,v)
				#include cTestEffect_attribsFile
				#undef ATTRIB
				return true;
			}

			// from cBaseNode
			/********************************************************************************************/
			const tBool set (const tDWord key, const string & value) {
				if (super::set (key, value)) return true;
				switch (key) {
					#define ATTRIB(n,t,v,d) ATTRIB_SET_S(n)
					#include cTestEffect_attribsFile
					#undef ATTRIB
				case '    ': return false;
				default: return false;
				}
				return false;
			}

			// from cBaseNode
			/********************************************************************************************/
			const tBool set (const tDWord key, const tFloat & value) {
				if (super::set (key, value)) return true;
				switch (key) {
					#define ATTRIB(n,t,v,d) ATTRIB_SET_N(n)
					#include cTestEffect_attribsFile
					#undef ATTRIB
				case '    ': return false;
				default: return false;
				}
				return false;
			}

			// from cBaseNode
			/********************************************************************************************/
			const tBool get (const tDWord key, string & value) const {
				if (super::get (key, value)) return true;
				switch (key) {
					#define ATTRIB(n,t,v,d) ATTRIB_GET(n)
					#include cTestEffect_attribsFile
					#undef ATTRIB
				case '    ': return false;
				default: return false;
				}
				return false;
			}

			// from cBaseNode
			/********************************************************************************************/
			const vector <tAttr> * getAttributeNames () const {
				if (attributeNames.size () == 0) {
					const vector <tAttr> * v = super::getAttributeNames ();
					attributeNames.insert (attributeNames.begin (), v->begin (), v->end ());
					#define ATTRIB(n,t,v,d) ATTRIB_ATTRIBNAMES(n,d)
					#include cTestEffect_attribsFile
					#undef ATTRIB
				}
				return &attributeNames;
			}

			// from cBaseNode
			/********************************************************************************************/
			const string getTypeName () const { return "caosGL::effects::cTestEffect"; }
		};
	}
}

// for node creation
#include <caosGL/core/cRegistry.h>
#include <caosGL/gfx/cNodeCreator.h>

class cTestEffectNodeCreator : public cNodeCreator {
public:
	cTestEffectNodeCreator () {
		name ("caosGL::effects::cTestEffect");
	}
	cBaseNode * createNode (const string n, cGroup * f) {
		return new caosGL::effects::cTestEffect (n,f);
	}
};
caosGL::core::cRegisterNodeCreator <cTestEffectNodeCreator> cTestEffectNodeCreatorInstance;

/**
 * The Catalyst of Design Software License, Version 1.0
 *
 * Copyright (c) 2002 Catalyst of Design (David Morris-Oliveros).  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by 
 *        Catalyst of Design (http://talsit.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "caosGL" and "Catalyst of Design" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact caosGL@talsit.org.
 *
 * 5. Products derived from this software may not be called "caosGL",
 *    nor may "caosGL" appear in their name, without prior written
 *    permission of Catalyst of Design.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL CATALYST OF DESIGN OR ITS 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 */
// eof